@@ -108,6 +108,9 @@ PhoneticEditor::processEscape (guint keyval, guint keycode,
     if (IBUS_Escape != keyval)
         return FALSE;
 
+    if (m_text.empty ())
+        return FALSE;
+
     if (cmshm_filter (modifiers) != 0)
         return TRUE;
 
@@ -152,7 +155,7 @@ PhoneticEditor::processSpace (guint keyval, guint keycode,
             assert (is_special_symbol (' '));
             if (m_props.modeFullWidth ()) {
                 String symbol;
-                convert_special_symbol (keyval, symbol);
+                check_result (convert_special_symbol (keyval, symbol));
                 commit (symbol);
             } else {
                 String symbol = ' ';
@@ -365,11 +368,15 @@ PhoneticEditor::processCandidateKey (guint keyval, guint keycode,
 
         case IBUS_Page_Up:
         case IBUS_KP_Page_Up:
+        case IBUS_Left:
+        case IBUS_KP_Left:
             pageUp ();
             return TRUE;
 
         case IBUS_Page_Down:
         case IBUS_KP_Page_Down:
+        case IBUS_Right:
+        case IBUS_KP_Right:
             pageDown ();
             return TRUE;
 
@@ -942,15 +949,19 @@ PhoneticEditor::getZhuyinCursor (void)
 }
 
 gboolean
-PhoneticEditor::insertSymbol (guint keyval)
+PhoneticEditor::insertSymbol (guint keyval, guint keycode, guint modifiers)
 {
     /* for symbols. */
     if (is_full_width_symbol (keyval)) {
 
         if (is_special_symbol (keyval)) {
             String choice = keyval;
-            if (m_props.modeFullWidth ())
-                assert (convert_special_symbol (keyval, choice));
+            if (m_props.modeFullWidth () ||
+                (modifiers & IBUS_SHIFT_MASK) ||
+                (keyval == IBUS_bracketleft ||
+                 keyval == IBUS_bracketright ||
+                 keyval == IBUS_apostrophe))
+                check_result (convert_special_symbol (keyval, choice));
 
             String lookup;
             int ch = find_lookup_key (choice);
@@ -962,9 +973,10 @@ PhoneticEditor::insertSymbol (guint keyval)
             return TRUE;
         }
 
-        if (m_props.modeFullWidth ()) {
+        if (m_props.modeFullWidth () ||
+            (modifiers & IBUS_SHIFT_MASK)) {
             String choice;
-            assert (convert_full_width_symbol (keyval, choice));
+            check_result (convert_full_width_symbol (keyval, choice));
 
             String lookup;
             int ch = find_lookup_key (choice);
@@ -994,7 +1006,9 @@ PhoneticEditor::insertNumbers (guint keyval)
                                         IBUS_KP_3, IBUS_KP_4,
                                         IBUS_KP_5, IBUS_KP_6,
                                         IBUS_KP_7, IBUS_KP_8,
-                                        IBUS_KP_9};
+                                        IBUS_KP_9,
+                                        IBUS_KP_Add, IBUS_KP_Subtract,
+                                        IBUS_KP_Multiply, IBUS_KP_Divide};
 
     /* for input pad numbers with num lock off. */
     static const guint num_lock_off[] = {IBUS_KP_Delete, IBUS_KP_Insert,
@@ -1002,14 +1016,18 @@ PhoneticEditor::insertNumbers (guint keyval)
                                          IBUS_KP_Next, IBUS_KP_Left,
                                          IBUS_KP_Begin, IBUS_KP_Right,
                                          IBUS_KP_Home, IBUS_KP_Up,
-                                         IBUS_KP_Prior};
+                                         IBUS_KP_Prior,
+                                         IBUS_KP_Add, IBUS_KP_Subtract,
+                                         IBUS_KP_Multiply, IBUS_KP_Divide};
 
     static const char numbers[] = {'.', '0',
                                    '1', '2',
                                    '3', '4',
                                    '5', '6',
                                    '7', '8',
-                                   '9'};
+                                   '9',
+                                   '+', '-',
+                                   '*', '/'};
 
     assert (G_N_ELEMENTS (num_lock_on) == G_N_ELEMENTS (numbers));
     assert (G_N_ELEMENTS (num_lock_off) == G_N_ELEMENTS (numbers));
